#!/bin/bash
# vim: dict=/usr/share/beakerlib/dictionary.vim cpt=.,w,b,u,t,i,k
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Description: kernelqa base lib/tools
#   Author: Jianhong Yin <jiyin@redhat.com>
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Copyright (c) 2013 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# library-prefix = qa
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

CASE_PATH=$PWD

if test -z "$OUTPUTFILE"; then
  OUTPUTFILE=/mnt/tests/$TEST/output.log
  rlLogInfo "output log will be stored in $OUTPUTFILE"
  mkdir -p $(dirname $OUTPUTFILE)
fi


# log # for backward compatible, sugest use rlLog in newcode.
log() { echo -e "$@" | tee -a "${OUTPUTFILE}"; }
    myecho() { log "$@"; }
    outputecho() { log "$@"; }

# abbreviation for report_result
test_pass() { report_result "$TEST/$1" "PASS"; }
test_fail() { report_result "$TEST/$1" "FAIL"; }
test_warn() { report_result "$TEST/$1" "WARN"; }
test_pass_exit() { report_result "$TEST/$1" "PASS"; exit 0; }
test_fail_exit() { report_result "$TEST/$1" "FAIL"; exit 1; }
test_warn_exit() { report_result "$TEST/$1" "WARN"; exit 1; }
testwakeup() { rhts-sync-set -s "$1"; }
testwaitfor() { rhts-sync-block -s "$@"; }

submit_log() {
	local log=$1
	[ -z "$TESTPATH" ] && return 0
	rhts_submit_log -S $RESULT_SERVER -T $TESTID -l $log
}

need_root() {
	uid=`id -u`
	if [ $uid -ne 0 ];then
		myecho "Need root to run this test"
		test_fail_exit
	fi
}

# is_thel* # for backward compatible, sugest use rlIsRHEL in newcode.
get_rhel_version() { echo -n `lsb_release -sr`; }
is_rhel() { grep -q "release $1" /etc/redhat-release; }
    is_rhel5() { is_rhel ${FUNCNAME#is_rhel}; }
    is_rhel6() { is_rhel ${FUNCNAME#is_rhel}; }
    is_rhel7() { is_rhel ${FUNCNAME#is_rhel}; }

disable_avc_check() {
	setenforce 0
	export AVC_ERROR=+no_avc_check
}

enable_avc_check() {
	setenforce 1
	echo "" >/var/log/audit/audit.log
}

#===============================================================================
# check whether this system use systemd
isSystemd() {
	test "$(cat /proc/1/comm)" = systemd;
}

#===============================================================================
#run command with limit time, avoid hang up
timeLimitRun() {
  local cmdId=timeLimitRun$RANDOM;
  local timeOut=$1
  local plist=""
  shift
  (for i in `seq 0 $timeOut`; do
      sleep 1;
      p=$(ps axf|grep "sh -c : $cmdId;"|grep -v grep);
      read pid nil <<<$p;
      #echo $p;
      [ $i -ge $timeOut -a -n "$pid" ] && {
          echo "{WARN} cmd{$@} timeOut(${timeOut}s)!"
          plist=$(pstree -paA $pid | sed  -e 's/^[^,]*,//' -e 's/ .*$//')
	  { kill $plist; usleep 100000; kill -9 $plist; } 2>/dev/null
          return 1;
      }
      [ -z "$pid" ] && break;
  done &)

  sh -c ": $cmdId; $*";
}
    timelimit_run() { timeLimitRun "$@"; }

#===============================================================================
run() {
    #[ -z "$BEAKERLIB_JOURNAL" -o -z "$BEAKERLIB_DIR" ] && {
    [ -z "$BEAKERLIB_DIR" ] && {
        rlJournalStart
        rlPhaseStartTest "$TEST"
    }
    GETOPT=$(getopt -q -o lcts -- "$@");
    eval set -- "$GETOPT";
    local DO_LOG=false;
    local DO_TAG=false;
    local DO_KEEP=false;
    local DO_CON=false;
    local TAG_OUT='';
    local TAG_ERR='';
    local LOG_FILE='';
    while true; do
        case "$1" in 
            -l)
                DO_LOG=true;
                [ -n "$LOG_FILE" ] || LOG_FILE=$( mktemp --tmpdir=$__INTERNAL_PERSISTENT_TMP );
                shift
            ;;
            -c)
                DO_LOG=true;
                DO_CON=true;
                [ -n "$LOG_FILE" ] || LOG_FILE=$( mktemp --tmpdir=$__INTERNAL_PERSISTENT_TMP );
                shift
            ;;
            -t)
                DO_TAG=true;
                TAG_OUT='STDOUT: ';
                TAG_ERR='STDERR: ';
                shift
            ;;
            -s)
                DO_KEEP=true;
                [ -n "$LOG_FILE" ] || LOG_FILE=$( mktemp --tmpdir=$__INTERNAL_PERSISTENT_TMP );
                shift
            ;;
            --)
                shift;
                break
            ;;
            *)
                shift
            ;;
        esac;
    done;
    if [ ! -e "$LOG_FILE" ]; then
        if $DO_LOG || $DO_KEEP; then
            rlFail "rlRun: Internal file creation failed";
            rlLogError "rlRun: Please report this issue to RH Bugzilla for Beakerlib component";
            rlLogError "rlRun: Turning off any -l, -c or -s options of rlRun";
            rlLogError "rlRun: Unless the test relies on them, rest of the test can be trusted.";
            DO_LOG=false;
            DO_KEEP=false;
        fi;
        LOG_FILE=/dev/null;
    fi;
    local command=$1;
    local expected_orig=${2:-0};
    local expected=${2:-0};
    local comment=${3:-"Running '$command'"};
    while echo "$expected" | grep --color=auto -q '[0-9]-[0-9]'; do
        local interval=$(echo "$expected" | sed "s/.*\(\<[0-9]\+-[0-9]\+\>\).*/\1/");
        if [ -z "$interval" ]; then
            rlLogWarning "rlRun: Something happened when getting interval, using '0-0'";
            interval='0-0';
        fi;
        local interval_a=$(echo "$interval" | cut -d '-' -f 1);
        local interval_b=$(echo "$interval" | cut -d '-' -f 2);
        if [ -z "$interval_a" -o -z "$interval_b" ]; then
            rlLogWarning "rlRun: Something happened when getting boundaries of interval, using '0' and '0'";
            interval_a=0;
            interval_b=0;
        fi;
        if [ $interval_a -gt $interval_b ]; then
            rlLogWarning "rlRun: First boundary have to be smaller then second one, using '$interval_b' and '$interval_b'";
            interval_a=$interval_b;
        fi;
        local replacement="$interval_a";
        let interval_a=$interval_a+1;
        for i in $(seq $interval_a $interval_b);
        do
            replacement="$replacement,$i";
        done;
        expected=$(echo "$expected" | sed "s/$interval/$replacement/");
    done;
    rlLogDebug "rlRun: Running command: $command";

    local RSBegin="================================================================================"
    local RSEnd="--------------------------------------------------------------------------------"
    if $DO_LOG || $DO_TAG || $DO_KEEP; then
        local UNBUFFER='';
        if set -o | grep --color=auto -q '^pipefail[[:space:]]'; then
            local pipefail=true;
            if set -o | grep --color=auto -q '^pipefail[[:space:]]*off$'; then
                set -o pipefail;
                local pipefail=false;
            fi;
        else
            rlLogWarning "rlRun: \`set -o pipefail\` not supported, exit code of command will not be checked correctly.";
        fi;

        echo "[$(date +%T) $USER@ ${PWD/$CASE_PATH/~~}]# $command" | tee -a $OUTPUTFILE
        eval "$UNBUFFER$command" 2> >(sed -u -e "s/^/$TAG_ERR/g" |
                tee -a $LOG_FILE) > >(sed -u -e "s/^/$TAG_OUT/g" | tee -a $LOG_FILE);
        local exitcode=$?;
        [ -n "$pipefail" ] && $pipefail || set +o pipefail;
        rlLogInfo "rlRun: command = '$command'; exitcode = $exitcode; expected = $expected";
    else
        echo "[$(date +%T) $USER@ ${PWD/$CASE_PATH/~~}]# $command" | tee -a $OUTPUTFILE;
        eval "$command" &> >(tee -a $OUTPUTFILE);
        local exitcode=$?;
    fi;
    if $DO_LOG || $DO_TAG || $DO_KEEP; then
        sync;
    fi;
    echo "$expected" | grep --color=auto -q "\<$exitcode\>";
    local result=$?;
    if $DO_LOG && ( ! $DO_CON || ( $DO_CON && [ $result -ne 0 ] ) ); then
        rlLog "Output of '$command':";
        rlLog "--------------- OUTPUT START ---------------";
        tail -n 50 "$LOG_FILE" | while read line; do
            rlLog "$line";
        done;
        rlLog "---------------  OUTPUT END  ---------------";
    fi;
    if $DO_KEEP; then
        rlRun_LOG=$LOG_FILE;
        export rlRun_LOG;
    else
        if $DO_LOG; then
            rm -i $LOG_FILE;
        fi;
    fi;
    [ "$expected_orig" != "-" ] && {
        rlLogDebug "rlRun: Command finished with exit code: $exitcode, expected: $expected_orig";
        __INTERNAL_ConditionalAssert "$comment" $result "(Expected $expected_orig, got $exitcode)";
    }
    echo "${RSEnd}"
    return $exitcode
}
[ -n "$xrun" ] && rlRun() { run "$@"; }

#===============================================================================
# use for multihost test case,
set_ssh_without_passwd() {
	which expect 2>/dev/null || yum install -y expect
	local pass=${1:-fedora};
	local host

	echo -e "${pass}\n${pass}" | passwd --stdin `id -nu`
	echo "{Info} change passwd to $pass finish"
	[ -n "$disableNoPasswd" ] && return 0

	echo "{Info} rhts_sync_set -s ChangePassword"
	rhts_sync_set -s ChangePassword

	echo "{Info} rhts_sync_block wait other machines change passwd done."
	rhts_sync_block -s ChangePassword ${CLIENTS//$HOSTNAME/} ${SERVERS//$HOSTNAME/}

	echo "{Info} ssh-keygen and ssh-copy-id to other machine ..."
	mkdir -p ~/.ssh && chmod 0700 ~/.ssh &&
		echo y | ssh-keygen  -f ~/.ssh/id_rsa -N ""
	for host in ${CLIENTS//$HOSTNAME/} ${SERVERS//$HOSTNAME/} $HOSTNAME; do
        # It takes a long time for s390x on RHEL-ALT to show the password prompt
        # So raise the timeout to 15 seconds
		expect -c  'set timeout 15
		spawn ssh-copy-id -i '"$host"'
		expect {
			"*yes/no" { send "yes\r"; exp_continue }
			"*assword:" { send "'"$pass"'\r" }
		}
		expect eof
		'
	done
	echo -e "{Info} set_ssh_without_passwd done."
	echo -e "------------------------------------------------"
}

# output the test env info:(Test name, distro, hostname, kernel, pkg versions)
envinfo() {
	(
	log "{INFO} Test env info:"
	log "------------------------------------------------"
	log "Time & CURDIR : [`date '+%F %T'` @$PWD]"
	log "Case Name     : $TEST"
	log '$HOSTNAME     : '$HOSTNAME
	log "Distro Info   : `lsb_release -sir` : $DISTRO_BUILD"
	log "NVR & host    : `uname -a`"
	log "cmdline       :"; cat /proc/cmdline | sed 's/^/\t/'
	log "Package Info  :"
	rpm -q `for p in $PKG_LIST "$@"; do echo $p; done|sort -u` 2>&1 |
		sed 's/^/\t/'

	[ -n "$FSTYP" ] && {
		log "FSTYP         : $FSTYP"
		log "SCRATCH_DEV   : $SCRATCH_DEV"
		log "SCRATCH_MNT   : $SCRATCH_MNT"
		log "SCRATCH_IMG   : $SCRATCH_IMG"
	}
	[ -n "$customPreView" ] && {
		log "\n{Info} Custom preview->[$customPreView]"
		eval "$customPreView"
	}
	echo -e "------------------------------------------------\n"
	) | tee -a "${OUTPUTFILE:-/dev/null}"
}

#===============================================================================

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#   Verification
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   This is a verification callback which will be called by
#   rlImport after sourcing the library to make sure everything is
#   all right. It makes sense to perform a basic sanity test and
#   check that all required packages are installed. The function
#   should return 0 only when the library is ready to serve.
pushd $(dirname $BASH_SOURCE) > /dev/null
QA_LIB_DIR="$PWD"
popd > /dev/null

qaLibraryLoaded() {
    local libDir=$QA_LIB_DIR
    local i
    for i in $libDir/lib-*.sh ; do
        if [ -r "$i" ]; then
            . "$i"
        fi
    done
    mkdir -p /usr/local/src /usr/local/bin
    \cp -af $libDir/src/* /usr/local/src/.
    \cp -af $libDir/tools/*  /usr/local/bin/.

    true
}

